/*
 * GPS Navigation http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 *
 * GPS Navigation is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * GPS Navigation is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import QtQuick.Layouts 1.1
import Ubuntu.Components 1.3
import Ubuntu.Components.Pickers 1.3
import Ubuntu.Components.Popups 1.3

Page {
    id: coordPage

    title: i18n.tr("Coordinates")
    anchors.fill: parent

    head {
        id: headPage
        backAction: Action {
            iconName: "back"
            text: i18n.tr("Back")
            onTriggered: {
                mainPageStack.pop(coordPage)
                mainPageStack.push(Qt.resolvedUrl("RoutePage.qml"))
            }
        }
        sections {
            model: [i18n.tr("Decimal"), i18n.tr("Sexagesimal")]
            selectedIndex: 0
        }
    }


    Column {
        id: colCoordDec
        visible: !coordPage.head.sections.selectedIndex
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        width: units.gu(28)
        height: units.gu(10)
     
        spacing: units.gu(2)
     
        Row {
            spacing: units.gu(1)
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: i18n.tr("Lat:")
                width: units.gu(5)
            }
     
            TextField {
                id: lat1
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: true
                maximumLength: 15
                width: units.gu(22)
                placeholderText: '51.506177'
            }
        }
     
        Row {
            spacing: units.gu(1)
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: i18n.tr("Long:")
                width: units.gu(5)
            }
     
            TextField {
                id: lng1
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: true
                maximumLength: 15
                width: units.gu(22)
                placeholderText: '-0.100236'
            }
        }
     
        Button {
            id: showCoordDec
            text: i18n.tr("Show on Map")
            width: colCoordDec.width
            anchors.topMargin: units.gu(5)
            anchors.horizontalCenter: parent.horizontalCenter
            color: UbuntuColors.green
            onClicked: {
                try {
                    var aux_lat = lat1.text;
                    var aux_lng = lng1.text;
                    if (!isNaN(aux_lat) && aux_lat.toString().indexOf('.') != -1 && !isNaN(aux_lng) && aux_lng.toString().indexOf('.') != -1 && aux_lat >= -90 && aux_lat <= 90 && aux_lng >= -180 && aux_lng <= 180) { // It's a float
                        mainPageStack.clickedLat = parseFloat(aux_lat).toFixed(5);
                        mainPageStack.clickedLng = parseFloat(aux_lng).toFixed(5);
                        if (mainPageStack.center_onpos === 2)
                            mainPageStack.center_onpos = 1;
                        mainPageStack.pop(coordPage);
                        mainPageStack.executeJavaScript(
                            "ui.markers_POI_set([{title: '', lat: " + mainPageStack.clickedLat + ", lng: " + mainPageStack.clickedLng + "}]);" 
                        );
                        mainPageStack.favPopup = false;
                        goThereActionPopover.show();
                    }
                    else {
                        PopupUtils.open(coordDecNotValid)
                    }
                }
                catch(e){
                    PopupUtils.open(coordDecNotValid)
                }
            }
        }

        Component {
            id: coordDecNotValid
            Dialog {
                id: dialogue
                title: i18n.tr("Coordinates are not valid")
                text: i18n.tr("Enter valid decimal coordinates\n\nExpected format is:") + "\n51.506177\n-0.100236"
                Button {
                     text: i18n.tr("Close")
                     onClicked: PopupUtils.close(dialogue)
                }
            }
        }
    }
    
    
    Column {
        id: colCoordPolar
        visible: coordPage.head.sections.selectedIndex
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        width: units.gu(35)
        height: units.gu(10)
    
        spacing: units.gu(2)
    
        Row {
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: i18n.tr("Lat:")
                width: units.gu(5)
            }
            
            TextField {
                id: lat2a
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(6)
                placeholderText: '51'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: "º"
                width: units.gu(1)
            }

            TextField {
                id: lat2b
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(8)
                placeholderText: '30'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: "'"
                width: units.gu(1)
            }

            TextField {
                id: lat2c
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(8)
                placeholderText: '22.23'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: "\""
                width: units.gu(1)
            }

            TextField {
                id: lat2d
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(5)
                maximumLength: 1
                placeholderText: 'N'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                width: units.gu(2)
            }
        }
     
        Row {
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: i18n.tr("Long:")
                width: units.gu(5)
            }
            
            TextField {
                id: lng2a
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(6)
                placeholderText: '0'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: "º"
                width: units.gu(1)
            }

            TextField {
                id: lng2b
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(8)
                placeholderText: '6'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: "'"
                width: units.gu(1)
            }

            TextField {
                id: lng2c
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(8)
                placeholderText: '0.84'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                text: "\""
                width: units.gu(1)
            }

            TextField {
                id: lng2d
                anchors.verticalCenter: parent.verticalCenter
                hasClearButton: false
                width: units.gu(5)
                maximumLength: 1
                placeholderText: 'W'
            }
            Label {
                anchors.verticalCenter: parent.verticalCenter
                width: units.gu(2)
            }
        }

        Button {
            id: showCoordPolar
            text: i18n.tr("Show on Map")
            width: colCoordPolar.width
            anchors.topMargin: units.gu(5)
            anchors.horizontalCenter: parent.horizontalCenter
            color: UbuntuColors.green
            onClicked: {
                try {
                    var aux_lat_day = parseInt(lat2a.text);
                    var aux_lat_min = parseFloat(lat2b.text);
                    var aux_lat_sec = lat2c.text === "" ? 0 : parseFloat(lat2c.text);
                    var aux_lat_dir = lat2d.text.toUpperCase();
                    var aux_lng_day = parseInt(lng2a.text);
                    var aux_lng_min = parseFloat(lng2b.text);
                    var aux_lng_sec = lng2c.text === "" ? 0 : parseFloat(lng2c.text);
                    var aux_lng_dir = lng2d.text.toUpperCase();
                    
                    if ((!isNaN(aux_lat_day) && !isNaN(aux_lat_min) && !isNaN(aux_lat_sec) && (aux_lat_dir === 'S' || aux_lat_dir === 'N')) &&
                        (!isNaN(aux_lng_day) && !isNaN(aux_lng_min) && !isNaN(aux_lng_sec) && (aux_lng_dir === 'W' || aux_lng_dir === 'E'))) {
                        var aux_lat = aux_lat_day + aux_lat_min/60 + aux_lat_sec/(60*60);
                        if (aux_lat_dir === "S" || aux_lat_dir === "W")
                            aux_lat = aux_lat * -1;
                        
                        var aux_lng = aux_lng_day + aux_lng_min/60 + aux_lng_sec/(60*60);
                        if (aux_lng_dir === "S" || aux_lng_dir === "W")
                            aux_lng = aux_lng * -1;
                        
                        if (aux_lat >= -90 && aux_lat <= 90 && aux_lng >= -180 && aux_lng <= 180) {
                            mainPageStack.clickedLat = parseFloat(aux_lat).toFixed(5);
                            mainPageStack.clickedLng = parseFloat(aux_lng).toFixed(5);
                            if (mainPageStack.center_onpos === 2)
                                mainPageStack.center_onpos = 1;
                            mainPageStack.pop(coordPage);
                            mainPageStack.executeJavaScript(
                                "ui.markers_POI_set([{title: '', lat: " + mainPageStack.clickedLat + ", lng: " + mainPageStack.clickedLng + "}]);" 
                            );
                            mainPageStack.favPopup = false;
                            goThereActionPopover.show();
                        }
                        else {
                            PopupUtils.open(coordPolarNotValid)
                        }
                    }
                    else {
                        PopupUtils.open(coordPolarNotValid)
                    }
                }
                catch(e){
                    PopupUtils.open(coordPolarNotValid)
                }
            }
        }

        Component {
            id: coordPolarNotValid
            Dialog {
                id: dialogue
                title: i18n.tr("Coordinates are not valid")
                text: i18n.tr("Enter valid sexagesimal coordinates\n\nExpected format is:") + "\n51° 30' 22.23'' N\n0° 6' 0.84'' W"
                Button {
                     text: i18n.tr("Close")
                     onClicked: PopupUtils.close(dialogue)
                }
            }
        }
    }
}

