/*
 * uNav http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan https://launchpad.net/~nik90
 *
 * uNav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * uNav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import QtQuick.LocalStorage 2.0
import "js/utils.js" as QmlJs
import "js/db.js" as UnavDB

Page {
    id: favoritesPage

    property string lat
    property string lng
    property string favName
    property bool isAddedFromPopup: false

    Component.onCompleted: {
        if (isAddedFromPopup) {
            addPOIFromPopup()
        }
    }

    header: standardHeader
    anchors.fill: parent

    function addPOIFromPopup() {
        if (favoritesPage.favName) {
            var exist_fav = UnavDB.getFavorite(favoritesPage.favName);
            if (exist_fav[0] === null || exist_fav[1] === null) {
                UnavDB.saveFavorite(favoritesPage.favName, favoritesPage.lat, favoritesPage.lng)
                favoritesModel.initialize()
            }
            else {
                header = addHeader
                favNameField.text = favoritesPage.favName
                PopupUtils.open(confirmOverwrite);
            }
        }
        favoritesPage.isAddedFromPopup = false
    }

    PageHeader {
        id: standardHeader
        title: i18n.tr("Favorites")
        visible: header === standardHeader
        flickable: favoritesListView
        leadingActionBar.actions: Action {
            iconName: "back"
            text: i18n.tr("Back")
            shortcut: "Escape"
            enabled: header === standardHeader
            onTriggered: {
                mainPageStack.pop(favoritesPage)
                mainPageStack.push(Qt.resolvedUrl("RoutePage.qml"))
            }
        }
        trailingActionBar.actions: Action {
            id: actionButton
            iconName: "add"
            shortcut: "Ctrl+N"
            text: i18n.tr("Add Favorite")
            onTriggered: {
                addActionList.show()
            }
        }
    }

    PageHeader {
        id: addHeader
        visible: header === addHeader
        flickable: favoritesListView
        leadingActionBar.actions:Action {
            iconName: "back"
            text: i18n.tr("Back")
            shortcut: "Escape"
            enabled: header === addHeader
            onTriggered: {
                header = standardHeader
                favNameField.text = ""
            }
        }
        contents: TextField {
            id: favNameField
            width: parent.width
            anchors.verticalCenter: parent.verticalCenter
            inputMethodHints: Qt.ImhNoPredictiveText
            hasClearButton: true
            placeholderText: i18n.tr("Insert a favorite name")
            text: (favoritesPage.favName !== "" && favoritesPage.favName !== i18n.tr("Current Position")) ? favoritesPage.favName : ""
            onTriggered: {
                if (text.length > 0 && favNameField.text !== "") {
                    var exist_fav = UnavDB.getFavorite(favNameField.text);
                    if (exist_fav[0] === null || exist_fav[1] === null) {
                        UnavDB.saveFavorite(favNameField.text, favoritesPage.lat, favoritesPage.lng)
                        favoritesModel.initialize()
                        favoritesPage.header = standardHeader
                    } else {
                        PopupUtils.open(confirmOverwrite)
                    }
                }
            }
        }
    }

    Label {
        id: statusLabel
        anchors.centerIn: parent
        visible: favoritesModel.count === 0
        text: i18n.tr("No favorites yet")
    }

    ListModel {
        id: favoritesModel
        Component.onCompleted: initialize()
        function initialize() {
            favoritesModel.clear();
            var res = UnavDB.getFavorites();
            for (var i = 0; i < res.rows.length; ++i) {
                favoritesModel.append({ name: res.rows.item(i).key,
                                        lat:  res.rows.item(i).lat,
                                        lng:  res.rows.item(i).lng
                                      });
            }
        }
    }

    ListView {
        id: favoritesListView

        model: favoritesModel
        anchors.fill: parent
        interactive: contentHeight > height

        delegate: ListItem {
            id: delegate

            height: favouriteDelegateLayout.height + divider.height
            leadingActions:  ListItemActions {
                actions: [
                    Action {
                        iconName: "delete"
                        onTriggered: {
                            UnavDB.removeFavorite(model.name);
                            favoritesModel.remove(index, 1)
                        }
                    }
                ]
            }

            trailingActions: ListItemActions {
                actions: [
                    Action {
                        iconName: "send"
                        onTriggered: {
                            mainPageStack.pop(favoritesPage);
                            mainPageStack.center_onpos = 2;
                            mainPageStack.routeState = 'yes';
                            mainPageStack.executeJavaScript("calc2coord("+ model.lat + "," + model.lng + ");");
                        }
                    },
                    Action {
                        iconName: "share"
                        onTriggered: {
                            PopupUtils.open(Qt.resolvedUrl("./Share.qml"), navApp, {"lat": model.lat, "lon": model.lng});
                        }
                    }
                ]
            }

            ListItemLayout {
                id: favouriteDelegateLayout
                title.text: model.name
                subtitle.visible: mainPageStack.center_onpos !== 0
                subtitle.text: QmlJs.formatDistance(QmlJs.calcPoiDistance(mainPageStack.currentLat, mainPageStack.currentLng, model.lat, model.lng, 10), navApp.settings.unit)
            }

            onClicked: {
                mainPageStack.pop(favoritesPage);
                if (navApp.settings.saveHistory) {
                    UnavDB.saveTofavHistory(model.name, model.lat, model.lng);
                }
                mainPageStack.executeJavaScript("ui.markers_POI_set([{title: \"" + model.name + "\", lat: " + model.lat + ", lng: " + model.lng + "}]);");
            }
        }
    }

    ActionSelectionPopover {
        id: addActionList

        width: units.gu(25)
        anchors {
            top: parent.top
            right: parent.right
            rightMargin: units.gu(0.5)
            topMargin: favoritesPage.header.height
        }

        delegate: ListItem {
            Label {
                text: action.text
                elide: Text.ElideMiddle
                opacity: action.enabled ? 1.0 : 0.5
                anchors { left: parent.left; right: parent.right; margins: units.gu(2); verticalCenter: parent.verticalCenter }
            }
        }

        actions: ActionList {
            Action {
                text: i18n.tr("Add Current Position")
                enabled: (mainPageStack.center_onpos !== 0 && mainPageStack.currentLat !== "null" && mainPageStack.currentLng !== "null")
                onTriggered: {
                    favoritesPage.lat = mainPageStack.currentLat;
                    favoritesPage.lng = mainPageStack.currentLng;
                    favoritesPage.header = addHeader
                    addActionList.hide();
                    favNameField.forceActiveFocus();
                }
            }
            Action {
                text: i18n.tr("Add Current Destination")
                enabled: (mainPageStack.routeState !== 'no')
                onTriggered: {
                    favoritesPage.lat = mainPageStack.endLat;
                    favoritesPage.lng = mainPageStack.endLng;
                    favoritesPage.header = addHeader
                    addActionList.hide();
                    favNameField.forceActiveFocus();
                }

            }
            Action {
                text: i18n.tr("Add From Map")
                onTriggered: mainPageStack.pop(favoritesPage);
            }
        }
    }


    Component {
        id: confirmOverwrite
        Dialog {
            id: dialogue
            title: i18n.tr("Overwrite Favorite")
            text: i18n.tr("There is already a favorite with that name. Do you want to overwrite it?")
            Column {
                width: parent.width
                Row {
                    anchors.horizontalCenter: parent.horizontalCenter
                    spacing: units.gu(2)
                    Button {
                        text: i18n.tr("Overwrite")
                        color: UbuntuColors.red
                        onClicked: {
                            UnavDB.saveFavorite(favNameField.text, favoritesPage.lat, favoritesPage.lng);
                            favoritesModel.initialize()
                            favoritesPage.header = standardHeader
                            PopupUtils.close(dialogue)
                        }
                    }
                    Button {
                        text: i18n.tr("Cancel")
                        onClicked: PopupUtils.close(dialogue)
                    }
                }
            }
        }
    }
}

