/*
 * uNav http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan https://launchpad.net/~nik90
 *
 * uNav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * uNav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import Ubuntu.Web 0.2
import Ubuntu.Connectivity 1.0
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import Qt.labs.settings 1.0
import com.canonical.Oxide 1.9 as Oxide
import Ubuntu.Content 1.1
import QtMultimedia 5.0
import QtFeedback 5.0
import QtSystemInfo 5.0
import "components"

MainView {
    id: navApp

    objectName: "navApp"
    applicationName: "navigator.costales"

    Component.onCompleted: {
        // Translations
        i18n.domain = "unav";
        i18n.bindtextdomain("unav", "nav/locales/mo");
        
        // AGPS warning
        if (navApp.settings.showAGPS && !navApp.settings.showTuto && !NetworkingStatus.wifiEnabled)
            PopupUtils.open(agps);
    }
    
    anchorToKeyboard: true

    width: units.gu(100)
    height: units.gu(70)

    property string applicationVersion: "0.57"
    property string mapUrl: "../nav/index.html"
    property string appUA: "Mozilla/5.0 (Linux; Android 5.0; Nexus 5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.2125.102 Mobile Safari/537.36"

    // persistent app settings:
    property var settings: Settings {
        property int soundIndications: 0 // 0 voice 1 notification 2 none
        property int unit: 0
        property int routingMode: 0 // 0 car, 1 walk, 2 bicycle
        property bool avoidTolls: false
        property bool alertRadars: false
        property bool legalRadarShow: true
        property string prevLat: ''
        property string prevLng: ''
        property int prevZoom: 9999
        property bool uiShowSpeed: false
        property bool saveHistory: true
        property bool headerVisible: true
        property bool showTuto: true
        property bool showAGPS: true
        property int defaultDistancePOI: 1
    }
    
    ScreenSaver {
        id: screenSaver
        screenSaverEnabled: !Qt.application.active
    }

    PageStack {
        id: mainPageStack

        Component.onCompleted: push(navigationPage)
        
        property string currentLat: "null"
        property string currentLng: "null"
        property string endLat: "null"
        property string endLng: "null"
        property string clickedLat: "null"
        property string clickedLng: "null"
        property string ptFromLat: "null"
        property string ptFromLng: "null"
        property string routeState: "no"
        property int center_onpos: 0 // 0 GPS off, 1 GPS on + not center, 2 GPS on + center
        property bool favPopup: false
        property bool onLoadingExecuted: false


        property string usContext: "messaging://"
        // Workaround: as long as map keeps a webcontainer this function handles js events to the webview.
        function executeJavaScript(code) {
            var req = webview.rootFrame.sendMessage(mainPageStack.usContext, "EXECUTE", {code: code});
            //req.onreply = function (msg) {
            //console.log(msg.str);
            //}
            req.onerror = function (code, explanation) {
                console.log("Error " + code + ": " + explanation)
            }
        }

        Page {
            id: navigationPage

            property bool buttonsEnabled: false

            anchors.fill: parent
            
            header: PageHeader {
                id: header

                title: "uNav"
                visible: navApp.settings.headerVisible

                leadingActionBar {
                    width: 0
                    visible: false
                }
                
                trailingActionBar {
                    numberOfSlots: 4
                    actions: [
                        Action {
                            id: actionFullScreen
                            iconSource: navigationPage.buttonsEnabled ? "../nav/img/header/restore.svg" : "../nav/img/header/restore_light.svg"
                            text: i18n.tr("Fullscreen")
                            enabled: navigationPage.buttonsEnabled
                            onTriggered: {
                                navApp.settings.headerVisible = !navApp.settings.headerVisible;
                            }
                        },

                        Action {
                            id: centerPosAction
                            iconSource: mainPageStack.center_onpos ? (mainPageStack.center_onpos === 1 ? "../nav/img/header/center_onpos.svg" : "../nav/img/header/center_onpos_light.svg") : navigationPage.buttonsEnabled ? "../nav/img/header/gps_off.svg" : "../nav/img/header/gps_off_light.svg"
                            text: i18n.tr("Center on Position")
                            enabled: navigationPage.buttonsEnabled && mainPageStack.center_onpos !== 2
                            onTriggered: {
                                mainPageStack.center_onpos = 2;
                                goThereActionPopover.hide();
                                mainPageStack.executeJavaScript("center_pos()");
                            }
                        },

                        Action {
                            id: routeAction
                            iconSource: navigationPage.buttonsEnabled ? "../nav/img/header/send.svg" : "../nav/img/header/send_light.svg"
                            text: i18n.tr("Route")
                            enabled: navigationPage.buttonsEnabled
                            onTriggered: {
                                mainPageStack.push(Qt.resolvedUrl("RoutePage.qml"))
                            }
                        },

                        Action {
                            id: actionSettings
                            iconSource: navigationPage.buttonsEnabled ? "../nav/img/header/settings.svg" : "../nav/img/header/settings_light.svg"
                            text: i18n.tr("Settings")
                            enabled: navigationPage.buttonsEnabled
                            onTriggered: mainPageStack.push(Qt.resolvedUrl("SettingsPage.qml"))
                        },

                        Action {
                            id: actionInfo
                            iconSource: navigationPage.buttonsEnabled ? "../nav/img/header/info.svg" : "../nav/img/header/info_light.svg"
                            text: i18n.tr("About")
                            enabled: navigationPage.buttonsEnabled
                            onTriggered: mainPageStack.push(Qt.resolvedUrl("AboutPage.qml"))
                        }
                    ]
                }
            }

            /**
            Workaround:
            QML Map Element currently does not support flicking. This makes it unusable to this app so far.
            As long as this is to supported, the map view falls back to a html5 container that renders the navigation.
            Signals from qml are sent via eventhandlers in oxide via userScript.
            The panel stays in html5, too, for ease of integration.
            **/

            WebContext {
                id: webcontext
                userAgent: appUA
                userScripts: [
                    Oxide.UserScript {
                        context: mainPageStack.usContext
                        url: Qt.resolvedUrl("js/oxide.js")
                    }
                ]
            }

            WebView {
                id: webview
                anchors.fill: parent
                z: -6
                context: webcontext
                url: mapUrl
                preferences.allowFileAccessFromFileUrls: true
                preferences.allowUniversalAccessFromFileUrls: true
                preferences.appCacheEnabled: true
                preferences.javascriptCanAccessClipboard: true
                preferences.javascriptEnabled: true
                //not used right now:
                //filePicker: filePickerLoader.item

                // get in-webcontainer click events (e.g. click on map)
                // and coordinates to handle them in qml ui:

                onNavigationRequested:{
                    // We could start to clean this up a bit by writing functions on that...
                    var url = request.url.toString().split("?");
                    var params = url[1].split("/");
                    
                    switch (url[0]) {
                        case "http://go/":
                            Qt.openUrlExternally(url[1]);
                            break;
                        
                        case "http://get_routeinfo_list/":
                            var dec_routeList = decodeURIComponent(url[1]);
                            mainPageStack.push(Qt.resolvedUrl("RouteInfoListPage.qml"), { routeList: JSON.parse(dec_routeList) });
                            navApp.settings.headerVisible = true;
                            break;
                        
                        case "http://route_status/":
                            mainPageStack.currentLat = params[0];
                            mainPageStack.currentLng = params[1];
                            mainPageStack.endLat = params[2];
                            mainPageStack.endLng = params[3];
                            mainPageStack.routeState = params[4];
                            break;
                        
                        case "http://clicked_on_map/":
                            mainPageStack.clickedLat = params[0];
                            mainPageStack.clickedLng = params[1];
                            
                            if (params[2] !== "none")
                                goThereActionPopover.poiName = decodeURI(params[2]);
                            else
                                goThereActionPopover.poiName = "";
                                
                            goThereActionPopover.website = '';
                            goThereActionPopover.phone = '';
                            if (params[3] !== 'none') {
                                goThereActionPopover.website = params[3].replace(/ /g, '/');
                                goThereActionPopover.website = goThereActionPopover.website.replace(/ç/g, '?');
                            }
                            if (params[4] !== 'none')
                                goThereActionPopover.phone = params[4];
                                
                            if (params[5] === 'nofollow' && mainPageStack.center_onpos === 2)
                                mainPageStack.center_onpos = 1;
                            mainPageStack.favPopup = false;
                            goThereActionPopover.show();
                            break;
                        
                        case "http://hide_popup/":
                            goThereActionPopover.hide();
                            break;
                            
                        case "http://set_center_onpos/":
                            if (params[0] === '2')
                                mainPageStack.center_onpos = 2;
                            else
                                mainPageStack.center_onpos = 1;
                            break;
                            
                        case "http://set_ui_speed/":
                            if (params[0] === 'true')
                                navApp.settings.uiShowSpeed = true;
                            else
                                navApp.settings.uiShowSpeed = false;
                            break;
                            
                        case "http://cancel_route/":
                            mainPageStack.routeState = 'no';
                            break;
                        
                        case "http://save_pos/":
                            navApp.settings.prevLat = params[0];
                            navApp.settings.prevLng = params[1];
                            navApp.settings.prevZoom = params[2];
                            break;
                    }
                    
                    request.action = Oxide.NavigationRequest.ActionReject;
                }

                Component.onCompleted: {
                    preferences.localStorageEnabled = true
                }

                onLoadingStateChanged: {
                    if (!loading && !mainPageStack.onLoadingExecuted) {
                        mainPageStack.onLoadingExecuted = true;
                        //send saved Setting states:
                        mainPageStack.executeJavaScript("settings.set_sound(" + navApp.settings.soundIndications + ");");
                        mainPageStack.executeJavaScript("settings.set_unit(\'" + ( navApp.settings.unit === 0 ? "km" : "mi" ) +"\');");
                        mainPageStack.executeJavaScript("ui.set_scale_unit(\'" + ( navApp.settings.unit === 0 ? "km" : "mi" ) +"\');");
                        mainPageStack.executeJavaScript("settings.set_routing_mode(" + navApp.settings.routingMode + ");");
                        mainPageStack.executeJavaScript("settings.set_avoid_tolls(" + navApp.settings.avoidTolls + ");");
                        mainPageStack.executeJavaScript("settings.set_alert_radars(" + navApp.settings.alertRadars + ");");
                        mainPageStack.executeJavaScript("settings.set_ui_speed(" + navApp.settings.uiShowSpeed + ");");
                        
                        // Center map in last position
                        if (navApp.settings.prevLat !== '' && navApp.settings.prevLng !== '' && navApp.settings.prevLat !== null && navApp.settings.prevLng !== null && navApp.settings.prevZoom !== 9999) {
                            mainPageStack.executeJavaScript("map.getView().setCenter(ol.proj.transform([" + navApp.settings.prevLng + "," + navApp.settings.prevLat + "], 'EPSG:4326', 'EPSG:3857')); map.getView().setZoom(" + navApp.settings.prevZoom + ");");
                        }
                        
                        // Hack: If user click so fast in buttons, app breaks sometimes
                        navigationPage.buttonsEnabled = true;
                        
                        // Catching urls
                        if (Qt.application.arguments[1].toString().indexOf('geo:') > -1 || Qt.application.arguments[1].toString().indexOf('http://unav-go.github.io') > -1) {
                            try {
                                if (Qt.application.arguments[1].toString().indexOf('geo:') > -1) {
                                    var go_params = Qt.application.arguments[1].toString().split(':')[1];
                                    if (go_params.toString().indexOf(';') > -1)
                                        go_params = go_params.split(';')[0];
                                    var aux_lat = go_params.split(',')[0];
                                    var aux_lng = go_params.split(',')[1];
                                }
                                else {
                                    var go_params = Qt.application.arguments[1].toString().split('?')[1];
                                    var aux_lat = go_params.split(',')[0].replace('p=', '');
                                    var aux_lng = go_params.split(',')[1];
                                }
                                
                                if (!isNaN(aux_lat) && aux_lat.toString().indexOf('.') != -1 && !isNaN(aux_lng) && aux_lng.toString().indexOf('.') != -1 && aux_lat >= -90 && aux_lat <= 90 && aux_lng >= -180 && aux_lng <= 180) { // It's a float
                                    mainPageStack.clickedLat = parseFloat(aux_lat);
                                    mainPageStack.clickedLng = parseFloat(aux_lng);
                                    if (mainPageStack.center_onpos === 2)
                                        mainPageStack.center_onpos = 1;
                                    mainPageStack.executeJavaScript("ui.markers_POI_set([{title: \"" + i18n.tr("Shared Position") + "\", lat: " + mainPageStack.clickedLat + ", lng: " + mainPageStack.clickedLng + "}]);");
                                    mainPageStack.favPopup = false;
                                    goThereActionPopover.show();
                                }
                            }
                            catch(e){}
                        }
                        else if (navApp.settings.showTuto) {
                            navApp.settings.showTuto = false;
                            mainPageStack.push(Qt.resolvedUrl("tuto/WelcomeWizard.qml"));
                        }
                    }
                }

                onGeolocationPermissionRequested: { request.allow() }
            }

            Connections {
                target: UriHandler
                onOpened: {
                    if (uris.length === 0 ) {
                        return;
                    }
                    // Catching urls
                    if (uris[0].toString().indexOf('geo:') > -1 || uris[0].toString().indexOf('http://unav-go.github.io') > -1) {
                        try {
                            if (uris[0].toString().indexOf('geo:') > -1) {
                                var go_params = uris[0].toString().split(':')[1];
                                if (go_params.toString().indexOf(';') > -1)
                                    go_params = go_params.split(';')[0];
                                var aux_lat = go_params.split(',')[0];
                                var aux_lng = go_params.split(',')[1];
                            }
                            else {
                                var go_params = uris[0].toString().split('?')[1];
                                var aux_lat = go_params.split(',')[0].replace('p=', '');
                                var aux_lng = go_params.split(',')[1];
                            }
                            
                            if (!isNaN(aux_lat) && aux_lat.toString().indexOf('.') != -1 && !isNaN(aux_lng) && aux_lng.toString().indexOf('.') != -1 && aux_lat >= -90 && aux_lat <= 90 && aux_lng >= -180 && aux_lng <= 180) { // It's a float
                                mainPageStack.clickedLat = parseFloat(aux_lat);
                                mainPageStack.clickedLng = parseFloat(aux_lng);
                                if (mainPageStack.center_onpos === 2)
                                    mainPageStack.center_onpos = 1;
                                mainPageStack.executeJavaScript("ui.markers_POI_set([{title: \"" + i18n.tr("Shared Position") + "\", lat: " + mainPageStack.clickedLat + ", lng: " + mainPageStack.clickedLng + "}]);");
                                mainPageStack.favPopup = false;
                                goThereActionPopover.show();
                            }
                        }
                        catch(e){}
                    }
                }
            }

            Row {
                id: actionButtonRow

                anchors { right: parent.right; top: parent.top; margins: units.gu(1) }

                ActionIcon {
                    icon.name: "send"
                    visible: !header.visible
                    enabled: navigationPage.buttonsEnabled
                    onClicked: mainPageStack.push(Qt.resolvedUrl("RoutePage.qml"))
                }

                ActionIcon {
                    icon.source: mainPageStack.center_onpos ? (mainPageStack.center_onpos === 1 ? "../nav/img/header/center_onpos.svg"
                                                                                                : "../nav/img/header/center_onpos_light.svg")
                                                            : navigationPage.buttonsEnabled ? "../nav/img/header/gps_off.svg"
                                                                                            : "../nav/img/header/gps_off_light.svg"
                    visible: !header.visible
                    enabled: navigationPage.buttonsEnabled && mainPageStack.center_onpos !== 2
                    onClicked: {
                        mainPageStack.center_onpos = 2;
                        goThereActionPopover.hide();
                        mainPageStack.executeJavaScript("center_pos()");
                    }
                }

                ActionIcon {
                    icon.source: navigationPage.buttonsEnabled ? "../nav/img/header/fullscreen.svg"
                                                               : "../nav/img/header/fullscreen_light.svg"
                    visible: !header.visible
                    enabled: navigationPage.buttonsEnabled
                    onClicked: navApp.settings.headerVisible = !navApp.settings.headerVisible;
                }

                ActionIcon {
                    icon.name: "settings"
                    visible: !header.visible
                    enabled: navigationPage.buttonsEnabled
                    onClicked: mainPageStack.push(Qt.resolvedUrl("SettingsPage.qml"))
                }
            }

            Column {
                id: zoomSplashColumn

                anchors {
                    right: parent.right
                    rightMargin: units.gu(1)
                    verticalCenter: parent.verticalCenter
                    verticalCenterOffset: units.gu(2)
                }

                ActionIcon {
                    icon.name: "zoom-in"
                    icon.width: units.gu(4)
                    visible: navigationPage.buttonsEnabled
                    onClicked: mainPageStack.executeJavaScript("custom_zoom(1)")
                }

                ActionIcon {
                    icon.name: "zoom-out"
                    icon.width: units.gu(4)
                    visible: navigationPage.buttonsEnabled
                    onClicked: mainPageStack.executeJavaScript("custom_zoom(-1)")
                }
            }

            ActionSelectionPopover {
                id: goThereActionPopover
                z: 100
                width: goThereActionPopover.poiName !== i18n.tr("Current Position") ? units.gu(37) : units.gu(31)
                height: mainPageStack.favPopup ? units.gu(21) : units.gu(16)
                property string poiName
                property string website
                property string phone
                anchors {
                    top: parent.top
                    left: parent.left
                    topMargin: units.gu(5)
                    leftMargin: units.gu(2)
                }

                delegate: Component {

                    Rectangle {
                        id: goThereRectangle
                        width: parent.width
                        height: goThereColumn.height + units.gu(2)
                        
                        Column {
                            id: goThereColumn
                            spacing: units.gu(1)
                            anchors {
                                centerIn: parent
                            }

                            Label {
                                id: poiLabel
                                text: goThereActionPopover.poiName
                                visible: goThereActionPopover.poiName !== ""
                                width: parent.width - units.gu(7)
                                maximumLineCount: 2
                                fontSize: "small"
                                horizontalAlignment: Text.AlignLeft
                                elide: Text.ElideRight
                                wrapMode: Text.WordWrap
                            }

                            AbstractButton {
                                id: nearByWebAbstractButton
                                anchors {
                                    right: parent.right
                                    rightMargin: units.gu(2)
                                }
                                visible: goThereActionPopover.website !== "" && goThereActionPopover.website.substring(0, 4) === "http"
                                Icon {
                                    id: nearByWebIcon
                                    width: units.gu(2)
                                    height: units.gu(2)
                                    name: "external-link"
                                }

                                onClicked: {
                                    Qt.openUrlExternally(goThereActionPopover.website)
                                }
                            }

                            AbstractButton {
                                id: nearByCallAbstractButton
                                anchors {
                                    right: parent.right
                                    rightMargin: (nearByWebAbstractButton.visible ? units.gu(5) : units.gu(2))
                                }
                                visible: goThereActionPopover.phone !== ""
                                Icon {
                                    id: nearByCallIcon
                                    width: units.gu(2)
                                    height: units.gu(2)
                                    name: "call-start"
                                }

                                onClicked: {
                                    Qt.openUrlExternally("tel:///"+ goThereActionPopover.phone)
                                }
                            }

                            Label {
                                text: i18n.tr("Coord:") + " " + parseFloat(mainPageStack.clickedLat).toFixed(5) + ", " + parseFloat(mainPageStack.clickedLng).toFixed(5)
                                fontSize: "small"
                                horizontalAlignment: Text.AlignHCenter
                                visible: !goThereActionPopover.poiName
                            }

                            Row {
                                spacing: 3

                                Button {
                                    id: goThereButton
                                    height: units.gu(5)
                                    width: units.gu(5)
                                    iconSource: "../nav/img/popup/send.svg"
                                    color: UbuntuColors.green
                                    visible: goThereActionPopover.poiName !== i18n.tr("Current Position") ? true : false
                                    onTriggered: {
                                        goThereActionPopover.hide();
                                        mainPageStack.center_onpos = 2;
                                        mainPageStack.routeState = 'yes';
                                        mainPageStack.executeJavaScript("calc2coord(" + mainPageStack.clickedLat + ", " + mainPageStack.clickedLng + ");");
                                    }
                                }

                                Button {
                                    id: addFavButton
                                    height: units.gu(5)
                                    width: units.gu(5)
                                    iconSource: mainPageStack.favPopup ? "../nav/img/popup/starred.svg" : "../nav/img/popup/non-starred.svg"
                                    color: UbuntuColors.greyGradient
                                    onTriggered: {
                                        mainPageStack.favPopup = !mainPageStack.favPopup;
                                        favNamePopup.text = '';
                                        if (mainPageStack.favPopup) {
                                            favNamePopup.forceActiveFocus();
                                        }
                                    }
                                }
                                
                                Button {
                                    id: poiButton
                                    height: units.gu(5)
                                    width: units.gu(5)
                                    iconSource: "../nav/img/popup/pois.svg"
                                    color: UbuntuColors.orange
                                    onTriggered: {
                                        mainPageStack.push(Qt.resolvedUrl("./PoiPage.qml"), {"fromPage": "Main.qml", "lat": mainPageStack.clickedLat, "lng": mainPageStack.clickedLng})
                                    }
                                }

                                Button {
                                    id: shareButton
                                    height: units.gu(5)
                                    width: units.gu(5)
                                    iconSource: "../nav/img/popup/share.svg"
                                    color: UbuntuColors.lightAubergine
                                    onTriggered: {
                                        PopupUtils.open(Qt.resolvedUrl("./Share.qml"), navApp, {"lat": mainPageStack.clickedLat, "lon": mainPageStack.clickedLng})
                                    }
                                }
                                
                                Button {
                                    id: ptFromButton
                                    height: units.gu(5)
                                    width: units.gu(5)
                                    iconSource: "../nav/img/popup/PtFrom.svg"
                                    visible: mainPageStack.ptFromLat === "null"
                                    color: UbuntuColors.darkGrey
                                    onTriggered: {
                                        goThereActionPopover.hide();
                                        mainPageStack.ptFromLat = mainPageStack.clickedLat;
                                        mainPageStack.ptFromLng = mainPageStack.clickedLng;
                                    }
                                }

                                Button {
                                    id: ptToButton
                                    height: units.gu(5)
                                    width: units.gu(5)
                                    iconSource: "../nav/img/popup/PtTo.svg"
                                    visible: mainPageStack.ptFromLat !== "null"
                                    color: UbuntuColors.darkGrey
                                    onTriggered: {
                                        goThereActionPopover.hide();
                                        mainPageStack.routeState = 'simulate_calculating';
                                        mainPageStack.executeJavaScript("simulate2coord(" + mainPageStack.ptFromLat + ", " + mainPageStack.ptFromLng + ", " + mainPageStack.clickedLat + ", " + mainPageStack.clickedLng + ");");
                                        mainPageStack.ptFromLat = "null";
                                    }
                                }
                            }
                            
                            TextField {
                                id: favNamePopup
                                width: parent.width
                                height: units.gu(4)
                                visible: mainPageStack.favPopup
                                inputMethodHints: Qt.ImhNoPredictiveText
                                placeholderText: i18n.tr("Favorite name")
                                onTriggered: {
                                    if (text.length > 0) {
                                        goThereActionPopover.hide();
                                        mainPageStack.push(Qt.resolvedUrl("FavoritesPage.qml"), {isAddedFromPopup: true, lat: mainPageStack.clickedLat, lng: mainPageStack.clickedLng, favName: favNamePopup.text});
                                    }
                                }
                            }
                        }
                    }
                }

                actions: ActionList {
                    Action {}
                }
            }
        }
    }

    Component {
        id: agps
        Dialog {
            id: agpsPopup
            contentHeight: units.gu(50)
            title: i18n.tr("AGPS is disabled")
            text: i18n.tr("Enabling WiFi (even not connected) in Meizu & bq devices will auto activate the AGPS feature.\nYou'll get a location in seconds!\n\nNote: In Nexus devices AGPS is not available. Acquiring a GPS signal could take a while.")
            Button {
                text: i18n.tr("Skip")
                color: UbuntuColors.red
                onClicked: PopupUtils.close(agpsPopup);
            }
            Button {
                text: i18n.tr("WiFi Settings")
                color: UbuntuColors.lightGrey
                onClicked: {
                    Qt.openUrlExternally("settings:///system/wifi");
                    PopupUtils.close(agpsPopup);
                }
            }
            ListItem {
                height: agpsPopup.height
                ListItemLayout {
                    id: rememberAgps
                    title.text: i18n.tr("Don't show this again")
                    Switch {
                        id: showAgpsSwitch
                        checked: !navApp.settings.showAGPS
                        onClicked: navApp.settings.showAGPS = !checked;
                    }
                }
            }
        }
    }

    Connections {
        target: Qt.application
        onStateChanged:
            if(Qt.application.state !== Qt.ApplicationActive) {
                goThereActionPopover.hide();
                mainPageStack.executeJavaScript("qml_save_pos();");
            }
    }
}
