/*
 * GPS Navigation http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan http://launchpad.net/~nik90
 *
 * GPS Navigation is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * GPS Navigation is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import QtQuick.XmlListModel 2.0
import QtQuick.LocalStorage 2.0
import "components"
import "js/PoiCategories.js" as Categories
import "js/db.js" as UnavDB

Page {
    id: poiPage

    property var fromPage
    property var lat
    property var lng
    property string unit: navApp.settings.unit === 0 ? "km" : "mi"
    property var factorList: ["1", "5", "15", "30", "50"]

    header: standardHeader
    anchors.fill: parent

    function goBackStandardMode() {
        mainPageStack.pop(poiPage);
        if (poiPage.fromPage === "RoutePage.qml")
            mainPageStack.push(Qt.resolvedUrl("RoutePage.qml"));
    }

    function goBackSearchMode() {
        categoryList.forceActiveFocus()
        poiPage.header = standardHeader
        poiSearchField.text = ""
    }

    Keys.onEscapePressed: {
        header === standardHeader ? goBackStandardMode()
                                  : goBackSearchMode()
    }

    PageHeader {
        id: standardHeader
        visible: poiPage.header === standardHeader
        title: i18n.tr("Nearby")
        leadingActionBar.actions: Action {
            iconName: "back"
            text: i18n.tr("Back")
            onTriggered: {
                goBackStandardMode()
            }
        }

        trailingActionBar.actions: Action {
            id: searchAction
            iconName: "find"
            text: i18n.tr("Search")
            shortcut: "Ctrl+F"
            onTriggered: {
                poiPage.header = searchHeader
                poiSearchField.forceActiveFocus()
            }
        }

        extension: Sections {
            id: distanceSections
            anchors {
                left: parent.left
                bottom: parent.bottom
            }

            model: [factorList[0]+unit, factorList[1]+unit, factorList[2]+unit, factorList[3]+unit, factorList[4]+unit]
            selectedIndex: navApp.settings.defaultDistancePOI
            onSelectedIndexChanged: navApp.settings.defaultDistancePOI = distanceSections.selectedIndex;
        }
    }

    PageHeader {
        id: searchHeader
        visible: poiPage.header === searchHeader
        leadingActionBar.actions: Action {
            iconName: "back"
            text: i18n.tr("Back")
            onTriggered: {
                goBackSearchMode()
            }
        }

        contents: TextField {
            id: poiSearchField
            width: parent.width
            anchors.verticalCenter: parent.verticalCenter
            inputMethodHints: Qt.ImhNoPredictiveText
            hasClearButton: true
        }
    }

    ListModel {
        id: categoryListModel
        function initialize() {
            categoryListModel.clear();
            Categories.data.forEach( function(category) {
                categoryListModel.append(category);
            });

            var res = UnavDB.getNearByHistory();
            var len = res.rows.length;
            for ( var i = 0; i < len; ++i) {
                categoryListModel.insert(
                    i,
                    {
                        theme: i18n.tr("Most recent"),
                        label: i18n.tr(res.rows.item(i).type),
                        en_label: res.rows.item(i).type,
                        clause: res.rows.item(i).clause
                    }
                );
            }
        }
        Component.onCompleted: initialize()
    }


    SortFilterModel {
        id: sortedCategoryListModel
        model: categoryListModel
        filter.property: "label"
        filter.pattern: poiPage.header === searchHeader ? RegExp(poiSearchField.text, "gi")
                                                        : RegExp("", "gi")
    }

    ListView {
        id: categoryList

        model: sortedCategoryListModel
        anchors.fill: parent
        anchors.topMargin: poiPage.header.height
        clip: true

        section.property: "theme"
        section.criteria: ViewSection.FullString
        section.labelPositioning: ViewSection.CurrentLabelAtStart + ViewSection.InlineLabels
        section.delegate: Rectangle {
            width: parent.width
            height: sectionHeader.height

            HeaderListItem {
                id: sectionHeader
                title: section
            }
        }

        delegate: ListItem {
            id: poiItem
            divider.visible: false
            height: poiItemLayout.height
            ListItemLayout {
                id: poiItemLayout
                title.text: label
                ProgressionSlot{}
            }
            onClicked: {
                poiSearchField.text = ""
                poiPage.header = standardHeader
                mainPageStack.push(Qt.resolvedUrl("PoiListPage.qml"), {
                    fromPage: poiPage,
                    lat: poiPage.lat,
                    lng: poiPage.lng,
                    poiType: model.label,
                    clause: model.clause,
                    geoDistFactor: Number(factorList[distanceSections.selectedIndex])
                });
                if (navApp.settings.saveHistory) {
                    UnavDB.saveToNearByHistory(model.en_label, model.clause);
                }
            }
        }
    }
}


