/*
 * uNav http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan https://launchpad.net/~nik90
 *
 * uNav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * uNav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3

Page {
    id: routePage

    header: PageHeader {
        title: i18n.tr("Route")
        flickable: flickable
    }

    Component.onCompleted: mainPageStack.executeJavaScript("qml_set_route_status();")

    Flickable {
        id: flickable

        anchors { fill: parent; margins: units.gu(2) }
        height: contentItem.childrenRect.height
        boundsBehavior: (contentHeight > routePage.height) ? Flickable.DragAndOvershootBounds : Flickable.StopAtBounds

        ListModel {
            id: routePageModel
            ListElement {section: 0 ; text: "Search"; iconName: "../nav/img/pages/route/search.svg" }
            ListElement {section: 0 ; text: "Favorites"; iconName: "../nav/img/pages/route/favorites.svg" }
            ListElement {section: 1 ; text: "Nearby"; iconName: "../nav/img/pages/route/nearby_enabled.svg"}
            ListElement {section: 2 ; text: "Nearby"; iconName: "../nav/img/pages/route/nearby_disabled.svg"}
            ListElement {section: 0 ; text: "Coordinates"; iconName: "../nav/img/pages/route/coordinates.svg"}
            ListElement {section: 0 ; text: "Share"; iconName: "../nav/img/pages/route/share.svg"}
            ListElement {section: 3 ; text: "Cancel Route"; iconName: "../nav/img/pages/route/cancel.svg" }
        }

        Grid {
            id: routePageGrid
            //from system-settings (lp:ubuntu-system-settings)
            property int itemWidth: units.gu(14)
            // The amount of whitespace, including column spacing
            property int space: parent.width - columns * itemWidth
            // The column spacing is 1/n of the left/right margins
            property int n: 1

            columnSpacing: space / ((2 * n) + (columns - 1))
            rowSpacing: units.gu(6)
            width: (columns * itemWidth) + columnSpacing * (columns - 1)
            anchors.horizontalCenter: parent.horizontalCenter
            columns: {
                var items = Math.floor(parent.width / itemWidth)
                var count = repeater.count
                return count < items ? count : items
            }

            Repeater {
                id: repeater

                model: routePageModel

                delegate: AbstractButton {
                    id: button

                    width: col.width
                    height: col.height
                    visible: (    (model.section === 0) ||
                              (model.section === 1 && mainPageStack.currentLat !== "null" && mainPageStack.currentLng !== "null") ||
                              (model.section === 2 && (mainPageStack.currentLat === "null" || mainPageStack.currentLng === "null")) ||
                              (model.section === 3 && mainPageStack.routeState !== 'no') )

                    UbuntuShape {
                        opacity: 0.15
                        visible: button.pressed
                        anchors { fill: col; margins: -units.gu(1) }
                        backgroundColor: UbuntuColors.darkGrey
                    }

                    Column {
                        id: col

                        spacing: units.gu(2)
                        width: units.gu (14)

                        Icon {
                            width: height
                            height: units.gu(4)
                            source: model.iconName
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        Label {
                            width: col.width
                            fontSize: "small"
                            text: i18n.tr(model.text)
                            horizontalAlignment: Text.AlignHCenter
                            wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                            anchors.horizontalCenter: parent.horizontalCenter
                        }
                    }

                    onClicked: {
                        if (model.text === "Nearby" && (mainPageStack.currentLat === "null" || mainPageStack.currentLng === "null")) {
                            // Nothing
                        }

                        else {
                            mainPageStack.pop(routePage)
                            if (model.text === "Favorites") {
                                mainPageStack.push(Qt.resolvedUrl("FavoritesPage.qml"))
                            } else if (model.text === "Search") {
                                mainPageStack.push(Qt.resolvedUrl("SearchPage.qml"))
                            } else if (model.text === "Nearby") {
                                mainPageStack.push(Qt.resolvedUrl("PoiPage.qml"), {"fromPage": "RoutePage.qml", "lat": mainPageStack.currentLat, "lng": mainPageStack.currentLng})
                            } else if (model.text === "Coordinates") {
                                mainPageStack.push(Qt.resolvedUrl("Coordinate.qml"))
                            } else if (model.text === "Share") {
                                mainPageStack.push(Qt.resolvedUrl("SharePage.qml"))
                            } else if (model.text === "Cancel Route") {
                                mainPageStack.routeState = 'no';
                                mainPageStack.executeJavaScript("click_cancel_route();")
                            }
                        }
                    }
                }
            }
        }
    }
}

