/*
 * uNav http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan https://launchpad.net/~nik90
 *
 * uNav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * uNav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import QtQuick.Layouts 1.1
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import Ubuntu.Components.ListItems 1.3 as ListItem

Page {
    id: sharePage
    title: i18n.tr("Share Position")

    head {
        id: headPage
        backAction: Action {
            iconName: "back"
            text: i18n.tr("Back")
            onTriggered: {
                mainPageStack.pop(sharePage)
                mainPageStack.push(Qt.resolvedUrl("RoutePage.qml"))
            }
        }
    }

    Flickable {
        id: flickable
        anchors.fill: parent
        anchors.topMargin: units.gu(6)
        height: contentItem.childrenRect.height
        boundsBehavior: (contentHeight > sharePage.height) ? Flickable.DragAndOvershootBounds : Flickable.StopAtBounds

        ListModel {
            id: sharePageModel
            ListElement {section: 0 ; text: "Current"; source: "../nav/img/pages/share/current.svg" }
            ListElement {section: 1 ; text: "Destination"; source: "../nav/img/pages/share/destination.svg"}
            ListElement {section: 2 ; text: "From Map"; source: "../nav/img/pages/share/fromMap.svg" }
        }

        Grid {
            id: sharePageGrid
            //from system-settings (lp:ubuntu-system-settings)
            property int itemWidth: units.gu(14)
            // The amount of whitespace, including column spacing
            property int space: parent.width - columns * itemWidth
            // The column spacing is 1/n of the left/right margins
            property int n: 1

            columnSpacing: space / ((2 * n) + (columns - 1))
            rowSpacing: units.gu(6)
            width: (columns * itemWidth) + columnSpacing * (columns - 1)
            anchors.horizontalCenter: parent.horizontalCenter
            columns: {
                var items = Math.floor(parent.width / itemWidth)
                var count = repeater.count
                return count < items ? count : items
            }

            Repeater {
                id: repeater
                model: sharePageModel
                delegate: Component {
                    id: buttonComponent
                    AbstractButton {
                        id: button
                        width: col.width
                        visible: ((model.section === 0 && (mainPageStack.center_onpos !== 0 && mainPageStack.currentLat !== "null" && mainPageStack.currentLng !== "null")) ||
                                  (model.section === 1 && (mainPageStack.routeState !== 'no' && mainPageStack.endLat !== "null" && mainPageStack.endLng !== "null")) ||
                                  (model.section === 2))
                        onClicked: {
                            if (model.text === "Current") {
                                PopupUtils.open(Qt.resolvedUrl("./Share.qml"), navApp, {"lat": mainPageStack.currentLat, "lon": mainPageStack.currentLng})
                            } else if (model.text === "Destination") {
                                PopupUtils.open(Qt.resolvedUrl("./Share.qml"), navApp, {"lat": mainPageStack.endLat, "lon": mainPageStack.endLng})
                            } else if (model.text === "From Map") {
                                mainPageStack.pop(sharePage); // Show map
                            }
                        }

                        height: col.height

                        Column {
                            id: col
                            width: units.gu (14)
                            height: units.gu (8)
                            anchors.left: parent.left
                            anchors.right: parent.right

                            Icon {
                                id: icon
                                anchors.horizontalCenter: parent.horizontalCenter
                                width: height
                                height: units.gu(4)
                                source: model.source
                            }

                            Label {
                                anchors.horizontalCenter: parent.horizontalCenter
                                anchors.bottom: parent.bottom
                                text: i18n.tr(model.text)
                                width: col.width
                                horizontalAlignment: Text.AlignHCenter
                                fontSize: "small"
                                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                            }
                        }

                        UbuntuShape {
                            z: -1
                            visible: button.pressed
                            anchors{
                                fill: col
                                margins: -units.gu(0.25)
                            }
                            backgroundColor: UbuntuColors.darkGrey
                            opacity: 0.15
                        }
                    }
                }
            }
        }
    }
}
