/** used from geolib.js:
* Calculates the distance between two spots.
* This method is more simple but also far more inaccurate
*
* @param	object	Start position {latitude: 123, longitude: 123}
* @param	object	End position {latitude: 123, longitude: 123}
* @param	integer   Accuracy (in meters)
* @return   integer   Distance (in meters)
*/
function calcPoiDistance(cur_lat, cur_lng, poi_lat, poi_lng, accuracy) {
	accuracy = Math.floor(accuracy) || 1;
	var distance =
		Math.round(
			Math.acos(
				Math.sin(
					poi_lat * Math.PI / 180
				) *
				Math.sin(
					cur_lat * Math.PI / 180
				) +
				Math.cos(
					poi_lat * Math.PI / 180
				) *
				Math.cos(
					cur_lat * Math.PI / 180
				) *
				Math.cos(
					(cur_lng - poi_lng) * Math.PI / 180
				)
			) * 6378137 // Earth radius
		);
	return Math.floor(Math.round(distance/accuracy)*accuracy);
}

/**
* Function formatDistance(..): formats distance string
* dist: distance in m
* unit: 0...km
*		1...mi
**/
function formatDistance (dist, unit) {
	var m2km = 0.001
	var m2mi = 0.000621371192
	var m2yd = 1.0936133
	
	if ( unit === 1 ) // mi
		var distance = dist * m2yd < 1000 ? parseInt(dist * m2yd) + " yd" : parseInt(dist * m2mi) + "+ mi";
	else
		var distance = dist < 1000 ? parseInt(dist) + " m" : parseInt(dist * m2km) + "+ km" ;
		
	if (distance === '0 m' || distance === '0 yd')
        return i18n.tr("Current Location");
	else
		return distance;
}

/**
* Function getTranslators(): Return all Ubuntu translators
* translators: string with all translators
**/
function getTranslators (translators_string) {
	var all_translators = [];
	
	if (translators_string === '' || translators_string === 'translator-credits') {
		all_translators.push({
			name: "Ubuntu Translators Community",
			link: "https://translations.launchpad.net/+groups/ubuntu-translators"
		});
		return all_translators;
	}
	
	var translators = translators_string.split('\n');
	translators.forEach(function(translator) {
		if (translator.indexOf("https://launchpad.net") > -1 && translator.indexOf("costales https://launchpad.net/~costales") < 0) { // First string will be the header and don't include myself
			var translator_name = translator.split(' https://launchpad.net/~')[0].trim();
			var translator_link = 'https://launchpad.net/~' + translator.split('https://launchpad.net/~')[1];
			all_translators.push({
				name: translator_name,
				link: translator_link
			});
		}
	});
	
	return all_translators;
}
